; #############################################################################
; #############################################################################
; #############################################################################
TWI_INITIALISIERUNG:
.equ TWI_BIT_RATE							= 10	; 10	 Bit-Rate							
.equ TWI_PRESCALER							= 0		; 0		 Vorteiler: 0=1 1=4 2=16 3=64	

	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	STS TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	STS TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	STS TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	STS TWCR,temp
	; PULL-UP ein								
	sbi (TWI_SCL_PORT),(TWI_SCL_PIN_NR)
	sbi (TWI_SDA_PORT),(TWI_SDA_PIN_NR)
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; START-ROUTINE	(MASTER TRANSMITTER)										   
;																			   
; OUTPUT: 	FEHLER_CODE														   
;																			   
TWI_MT_START:
	WDR
	clr ERROR
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)	; START								
	STS TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1		
	LDS temp,TWSR								; Check TWSR						
	andi temp, 0xF8
	cpi temp,  0x08								; 0x08 START erfolgreich			
	breq TWI_MT_START_w
	cpi temp,  0x10								; 0x10 (repeated START)				
	breq TWI_MT_START_w
	rjmp TWI_MT_START_ERROR
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	LDS temp,(adr_TWI)
	; Load SLA + "W"  (W=0) 			
	LSL  temp									; 1 x links							
	STS TWDR, temp

	ldi temp, (1<<TWINT) | (1<<TWEN)			; senden - Clear TWINT 				
	STS TWCR, temp

	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1		

	LDS temp,TWSR								; Check TWSR						
	andi temp, 0xF8
	cpi temp,  0x18								; 0x18								
	brne TWI_MT_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_ERROR:
	mov ERROR, EINS
	LED_ROT_ON
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Ein BYTE zum SLAVE zu senden										   	   	   
; 																			   
; INPUT:	temp <= DATEN													   
; OUT: 		FEHLER_CODE		   												   
; 																			   
TWI_MT_TX_TEMP:
	tst ERROR								; Fehler =0 ?	
	breq TWI_MT_DATA_w						; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MT_DATA_w:
	STS TWDR, temp							; 	Load DATA						
	ldi temp, (1<<TWINT) | (1<<TWEN)		; senden - Clear TWINT 				
	STS TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	LDS temp,TWSR							; Check TWSR						
	andi temp, 0xF8
	cpi temp, 0x28							; 0x28								
	brne TWI_MT_DATA_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MT_DATA_ERROR:
	mov ERROR, EINS
	LED_ROT_ON
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUT: 		FEHLER_CODE		   												   
;																			   
TWI_MR_START:
	clr ERROR
	ldi temp, (1<<TWINT|1<<TWEA|1<<TWSTA|1<<TWEN)	; START 						
	STS TWCR, temp	
	rcall TWI_TWINT_wait							; warten, bis TWINT wieder = 1	
	LDS temp,TWSR									; Check TWSR 					
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich		
	breq TWI_MR_START_w
	cpi temp,  0x10									; 0x10 (repeated START)			
	breq TWI_MR_START_w
	rjmp TWI_MR_START_ERROR
;-------------------------------------------------------------------------------
TWI_MR_START_w:
	; Load SLA +"R" (R=1)								
	LDS temp,(adr_TWI)
	lsl  temp	; 1 x links
	ori temp, 0b00000001 
	STS TWDR, temp
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MR_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MR_START_ERROR:
	mov ERROR, EINS									
	LED_ROT_ON
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Bytes empfangen werden sollen										
;																				
; OUTPUT: temp1																	
; OUTPUT: FEHLER_CODE		   													
;																				
TWI_MR_DATA_TEMP1:
	tst ERROR
	brne TWI_MR_DATA_ERROR
;-------------------------------------------------------------------------------
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	STS TWCR, temp
	rcall TWI_TWINT_wait						; TWINT abfragen									
	LDS temp,TWSR								; Check TWSR										
	andi temp, 0xF8
	cpi temp, 0x50								; Data Byte empfangen- ACK zurckgesendet			
	brne TWI_MR_DATA_ERROR
	LDS temp1, TWDR								; DATEN einlesen									
	ret
;-------------------------------------------------------------------------------
TWI_MR_DATA_ERROR:
	mov ERROR, EINS	
	clr temp1
	LED_ROT_ON
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MR_DATA_LAST_TEMP1:
	tst ERROR
	brne TWI_MR_DATA_LAST_ERROR
;-------------------------------------------------------------------------------

	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	STS TWCR, temp

	rcall TWI_TWINT_wait						; TWINT abfragen									

	LDS temp,TWSR								; Check TWSR										
	andi temp, 0xF8
	cpi  temp, 0x58								; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MR_DATA_LAST_ERROR
	LDS temp1, TWDR								; DATEN einlesen									
	ret
;-------------------------------------------------------------------------------
TWI_MR_DATA_LAST_ERROR:
	mov ERROR, EINS	
	clr temp1
	LED_ROT_ON
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	STS TWCR, temp	
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	LED_GELB_ON
	LDS temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
	LED_GELB_OFF
ret
; #############################################################################
; #############################################################################
; #############################################################################



