; ##############################################################################
; ##############################################################################
; ##############################################################################

.equ 	SSD1306_ADR						= 60
.equ 	display_clk						= 0xF0		; 0x80
.equ	charge_pump						= 0x14


// Control byte
.equ	SSD1306_CONTROL_BYTE_CMD_SINGLE		= 0x80
.equ	SSD1306_CONTROL_BYTE_CMD_STREAM		= 0x00
.equ	SSD1306_CONTROL_BYTE_DATA_SINGLE	= 0xC0
.equ	SSD1306_CONTROL_BYTE_DATA_STREAM	= 0x40

// Fundamental commands (pg.28)
.equ	SSD1306_CMD_SET_CONTRAST			= 0x81    // follow with 0x7F
.equ	SSD1306_CMD_DISPLAY_RAM				= 0xA4
.equ	SSD1306_CMD_DISPLAY_ALLON			= 0xA5
.equ	SSD1306_CMD_DISPLAY_NORMAL			= 0xA6
.equ	SSD1306_CMD_DISPLAY_INVERTED		= 0xA7
.equ	SSD1306_CMD_DISPLAY_OFF				= 0xAE
.equ	SSD1306_CMD_DISPLAY_ON				= 0xAF

// Addressing Command Table (pg.30)
.equ	SSD1306_CMD_SET_LOWER_COLUMN		= 0x00
.equ	SSD1306_CMD_SET_HIGHER_COLUMN		= 0x10
.equ	SSD1306_CMD_SET_MEMORY_ADDR_MODE	= 0x20    // follow with 0x00 = HORZ mode = Behave like a KS108 graphic LCD
.equ	SSD1306_CMD_SET_PAGE_START_ADDRESS	= 0xB0    //~0xB7
.equ	SSD1306_CMD_SET_COLUMN_ADDRESS		= 0x21    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x7F = COL127
.equ	SSD1306_CMD_SET_PAGE_ADDRESS		= 0x22    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x07 = PAGE7
.equ	HORIZONTAL_ADDRESSING_MODE			= 0
.equ	VERTICAL_ADDRESSING_MODE			= 1
.equ	PAGE_ADDRESSING_MODE				= 2
// Hardware Config (pg.31)
.equ	SSD1306_CMD_SET_DISPLAY_START_LINE	= 0x40
.equ	SSD1306_CMD_SET_SEGMENT_REMAP_01	= 0xA0    
.equ	SSD1306_CMD_SET_SEGMENT_REMAP_02	= 0xA1    
.equ	SSD1306_CMD_SET_MUX_RATIO			= 0xA8    // follow with 0x3F = 64 MUX
.equ	SSD1306_CMD_SET_COM_OUT_NORMAL		= 0xC0
.equ	SSD1306_CMD_SET_COM_OUT_REMAPPED	= 0xC8    
.equ	SSD1306_CMD_SET_DISPLAY_OFFSET		= 0xD3    // follow with 0x00
.equ	SSD1306_CMD_SET_COM_PIN_MAP			= 0xDA    // follow with 0x12
.equ	SSD1306_CMD_NOP						= 0xE3    // NOP

// Timing and Driving Scheme (pg.32)
.equ	SSD1306_CMD_SET_DISPLAY_CLK_DIV		= 0xD5    // follow with 0x80
.equ	SSD1306_CMD_SET_PRECHARGE			= 0xD9    // follow with 0xF1
.equ	SSD1306_CMD_SET_VCOMH_DESELCT		= 0xDB    // follow with 0x30

// Charge Pump (pg.62)
.equ	SSD1306_CMD_SET_CHARGE_PUMP			= 0x8D    // follow with 0x14
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_OLED_INITIALISIERUNG:
	ldi temp,(SSD1306_ADR)										; TWI Adresse	
	STS(adr_TWI),temp
;-------------------------------------------------------------------------------
;	rcall	wait_20ms		; Pause
;-------------------------------------------------------------------------------
	rcall TWI_MT_START											; Start			
;-------------------------------------------------------------------------------
	ldi temp,(SSD1306_CONTROL_BYTE_CMD_STREAM)					; CONTROL Byte: CMD STREAM
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_DISPLAY_OFF)							; DISPLAY_OFF
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_NOP)									; NOP
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_MEMORY_ADDR_MODE)					; SET_MEMORY_ADDR_MODE
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(HORIZONTAL_ADDRESSING_MODE)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_PAGE_START_ADDRESS)				; SET_PAGE_START_ADDRESS (0xB0)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
;	ldi	temp,(SSD1306_CMD_SET_COM_OUT_NORMAL)					; SET_COM_REMAPPED (0xC0)
	ldi	temp,(SSD1306_CMD_SET_COM_OUT_REMAPPED)					; SET_COM_REMAPPED (0xC8)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_LOWER_COLUMN)						; SET_LOWER_COLUMN (0x00) -> SEG0
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_HIGHER_COLUMN)					; SET_HIGHER_COLUMN (0x10) -> SEG0
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_COLUMN_ADDRESS)					; SET_COLUMN_ADDRRSS
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0)												; Start Address
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(127)												; End Address
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	r17,(SSD1306_CMD_SET_PAGE_ADDRESS)						; SET_PAGE_ADDRRSS
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0)												; Start Address
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(7)												; End Address
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_DISPLAY_START_LINE)				; SET_DISPLAY_START_LINE (0x40) -> 0
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_DISPLAY_RAM)							; DISPLAY_RAM
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_SEGMENT_REMAP_02)					; SET_SEGMENT_REMAP_02 (0xA1)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_DISPLAY_NORMAL)						; DISPLAY_NORMAL
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_CONTRAST)							; SET_CONTRAST						
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(255)												; VALUE								
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_MUX_RATIO)						; SET_MUX_RATIO (0xA8)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0x3F)												; 0x3F = 63		
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_DISPLAY_OFFSET)					; SET_DISPLAY_OFFSET (0xD3)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0x00)												; (0x00)		
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_DISPLAY_CLK_DIV)					; SET_DISPLAY_CLK_DIV
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(display_clk)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_PRECHARGE)						; SET_PRECHARGE
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0x22)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_COM_PIN_MAP)						; SET_COM_PIN_MAP (0xDA)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0x12)												; (0x12)		
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_VCOMH_DESELCT)					; SET_VCOMH_DESELCT
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(0x20)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_SET_CHARGE_PUMP)						; SET_CHARGE_PUMP
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(charge_pump)
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi	temp,(SSD1306_CMD_DISPLAY_ON)							; DISPLAY_ON
	rcall TWI_MT_TX_TEMP
	rjmp TWI_STOP												; STOP		
; ##############################################################################
; ##############################################################################
; ##############################################################################
OLED_CLEAR:
	ldi ZL,low (adr_BILD+0)
	ldi ZH,high(adr_BILD+0)
	ldi XL,low (1024/16)
	ldi XH,high(1024/16)
OLED_CLEAR_s:
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	ST Z+,NULL
	sbiw XL,1
	brne OLED_CLEAR_s
	rjmp OLED_REFRESH
; ##############################################################################
; ##############################################################################
; ##############################################################################
OLED_HELL_TEMP1:
	rcall TWI_MT_START											; Start			
	ldi temp,(SSD1306_CONTROL_BYTE_CMD_STREAM)					; CONTROL Byte: CMD STREAM
	rcall TWI_MT_TX_TEMP
	ldi	temp,(SSD1306_CMD_SET_CONTRAST)							; SET_CONTRAST						
	rcall TWI_MT_TX_TEMP
	mov temp,temp1												; VALUE								
	rcall TWI_MT_TX_TEMP
	rjmp TWI_STOP												; STOP		
; ##############################################################################
; ##############################################################################
; ##############################################################################
OLED_READ_BYTE_TEMP1:
	call TWI_MR_START
	call TWI_MR_DATA_LAST_TEMP1
	rjmp TWI_STOP												; STOP		
; ##############################################################################
; ##############################################################################
; ##############################################################################
; horizontale polarisierte SRAM Bytes in vertikale OLED Bytes umwandeln			
OLED_REFRESH:
	rcall GO_HOME
;-------------------------------------------------------------------------------
	rcall TWI_MT_START
	ldi	temp,(SSD1306_CONTROL_BYTE_DATA_STREAM)	; CONTROL Byte: DATA STREAM
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	ldi ZL,low (adr_BILD+0)
	ldi ZH,high(adr_BILD+0)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+1*128)
	ldi ZH,high(adr_BILD+1*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+2*128)
	ldi ZH,high(adr_BILD+2*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+3*128)
	ldi ZH,high(adr_BILD+3*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+4*128)
	ldi ZH,high(adr_BILD+4*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+5*128)
	ldi ZH,high(adr_BILD+5*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+6*128)
	ldi ZH,high(adr_BILD+6*128)
	rcall OLED_REFRESH_ZEILE
	ldi ZL,low (adr_BILD+7*128)
	ldi ZH,high(adr_BILD+7*128)
	rcall OLED_REFRESH_ZEILE
	rjmp TWI_STOP												; STOP		
;-------------------------------------------------------------------------------
OLED_REFRESH_ZEILE:
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rcall OLED_REFRESH_BLOCK
	rjmp OLED_REFRESH_BLOCK
;-------------------------------------------------------------------------------
OLED_REFRESH_BLOCK:
	LD temp1,Z
	adiw ZL,16
	LD temp2,Z
	adiw ZL,16
	LD temp3,Z
	adiw ZL,16
	LD temp4,Z
	adiw ZL,16
	LD temp5,Z
	adiw ZL,16
	LD temp6,Z
	adiw ZL,16
	LD temp7,Z
	adiw ZL,16
	LD temp8,Z

	sbiw ZL,60	; -111
	sbiw ZL,51
;-------------------------------------------------------------------------------
	ldi temp9,8
OLED_REFRESH_BLOCK_s:
	LSR temp1
	ROR temp
	LSR temp2
	ROR temp
	LSR temp3
	ROR temp
	LSR temp4
	ROR temp
	LSR temp5
	ROR temp
	LSR temp6
	ROR temp
	LSR temp7
	ROR temp
	LSR temp8
	ROR temp
	rcall TWI_MT_TX_TEMP
	dec temp9
	brne OLED_REFRESH_BLOCK_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
OLED_VOLL:
	ldi ZL,low (adr_BILD+0)
	ldi ZH,high(adr_BILD+0)
	ldi XL,low (1024)
	ldi XH,high(1024)
OLED_VOLL_s:
	ST Z+,VOLL
	sbiw XL,1
	brne OLED_VOLL_s
	rjmp OLED_REFRESH
; ##############################################################################
; ##############################################################################
; ##############################################################################
GO_HOME:
	rcall TWI_MT_START									; Start	
	ldi	temp,(SSD1306_CONTROL_BYTE_CMD_STREAM)			; CONTROL Byte: CMD STREAM
	rcall TWI_MT_TX_TEMP
;-------------------------------------------------------------------------------
	clr temp
	ori	temp,(SSD1306_CMD_SET_PAGE_START_ADDRESS)		; SET_PAGE START_ADDRESS
	rcall TWI_MT_TX_TEMP

	ldi	temp,(SSD1306_CMD_SET_COLUMN_ADDRESS)			; SET_COLUMN_ADDRRSS	
	rcall TWI_MT_TX_TEMP

	ldi temp,0											; Start Address			
	rcall TWI_MT_TX_TEMP

	ldi	temp,127										; End Address			
	rcall TWI_MT_TX_TEMP

	ldi temp,(SSD1306_CMD_SET_DISPLAY_START_LINE)+(0)	; SET_DISPLAY_START_LINE
	rcall TWI_MT_TX_TEMP
	rjmp TWI_STOP										; STOP					
; ##############################################################################
; ##############################################################################
; ##############################################################################
OLED_TEST_CLEAR:
	STS(adr_STOP_UHR_L),NULL
	STS(adr_STOP_UHR_H),NULL
	rcall OLED_CLEAR
	LDS temp1,(adr_STOP_UHR_L)
	LDS temp2,(adr_STOP_UHR_H)
	STS(adr_STOP_UHR_ERGEBNIS_L),temp1
	STS(adr_STOP_UHR_ERGEBNIS_H),temp2

	FONT1
	ZEILE0
	ldi ZH, high(TEXT_CLEAR*2)	; TEXT 
	ldi ZL,  low(TEXT_CLEAR*2)
	call ASCII_TEXT_Z
	ZEILE1
	call ANZEIGE_STOPUHR_KURZ
	call OLED_REFRESH
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

