;															
; OLED2 und ATmega328p										
; Bernhard.Erfurt@gmx.de									
; 01/2021													
;															
; https://www.mikrocontroller.net/topic/511295#new			
;															

; INTERRUPT_ERROR			ROT+GELB+GRUEN blinkt			
; WDR ERROR					ROT+GELB blinkt					
; PROGRAMM CHECK ERROR		ROT+GRN blinkt					

; ATmega328p 									
.include "m328pdef.inc"
.include "MACROS.asm"
; KONSTANTEN									
.equ CPU_TAKT				= 8000000			; 8 MHz	
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; TWI-SDA										
.equ TWI_SDA_PIN_NR					= 4
.equ TWI_SDA_PIN					= PINC
.equ TWI_SDA_DDR					= DDRC
.equ TWI_SDA_PORT					= PORTC
; TWI-SCL										
.equ TWI_SCL_PIN_NR					= 5
.equ TWI_SCL_PIN					= PINC
.equ TWI_SCL_DDR					= DDRC
.equ TWI_SCL_PORT					= PORTC
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						= SRAM_START+0			; START Adresse 	

.equ adr_STOP_UHR_L					=SRAM_START+5
.equ adr_STOP_UHR_H					=SRAM_START+6
.equ adr_STOP_UHR_ERGEBNIS_L		=SRAM_START+7
.equ adr_STOP_UHR_ERGEBNIS_H		=SRAM_START+8

.equ adr_1ms						=SRAM_START+10
.equ adr_10ms						=SRAM_START+11
.equ adr_100ms						=SRAM_START+12
.equ adr_ss							=SRAM_START+13	; Sekunden					
.equ adr_mm							=SRAM_START+14	; Minuten					
.equ adr_hh							=SRAM_START+15	; Stunden

.equ adr_TWI						=SRAM_START+20

.equ adr_ADC_0_L					=SRAM_START+30
.equ adr_ADC_0_H					=SRAM_START+31
.equ adr_ADC_1_L					=SRAM_START+32
.equ adr_ADC_1_H					=SRAM_START+33
.equ adr_ADC_2_L					=SRAM_START+34
.equ adr_ADC_2_H					=SRAM_START+35
.equ adr_ADC_3_L					=SRAM_START+36
.equ adr_ADC_3_H					=SRAM_START+37

.equ adr_ADC_VBG_L					=SRAM_START+44
.equ adr_ADC_VBG_H					=SRAM_START+45
.equ adr_ADC_BETRIEBSSPANNUNG		=SRAM_START+46

.equ adr_DELTA_X					=SRAM_START+50
.equ adr_DELTA_Y					=SRAM_START+51

.equ adr_FONT						=SRAM_START+80	; 0=8x12 1=16x24			
.equ adr_CURSOR_ZEILE				=SRAM_START+81	; CURSOR TV-ZEILE vom CURSOR
.equ adr_CURSOR_SPALTE				=SRAM_START+82	; CURSOR-SPALTE (0..19)		

.equ adr_SRAM_CHECK_A				=SRAM_START+99
.equ adr_BILD						=SRAM_START+100
.equ adr_BILD_ENDE					=SRAM_START+100+1024
.equ adr_SRAM_CHECK_B				=SRAM_START+100+1025

.equ adr_SRAM_CHECK					=SRAM_START+1900	; SRAM CHECK muss immer "S" sein
; REGISTER										
.def HILFSREGISTER0		= R0
.def HILFSREGISTER1		= R1
.def X1					= R2
.def Y1					= R3
.def X2					= R4
.def Y2					= R5

.def ZL_KOPIE			= R10
.def ZH_KOPIE			= R11

.def ERROR				= R12
.def NULL 				= R13
.def EINS			 	= R14
.def VOLL 			  	= R15

.def temp 				= R16
.def temp1 				= R17
.def temp2 				= R18
.def temp3 				= R19
.def temp4 				= R20
.def temp5 				= R21
.def temp6 				= R22
.def temp7 				= R23
.def temp8 				= R24
.def temp9 				= R25


.cseg							;Beginn eines Code-Segmentes	
.org 0							;Startadresse=0					
.org 0x0000 jmp RESET 			; Reset Handler										
.org 0x0002 jmp INTERRUPT_ERROR ; EXT_INT0 		; IRQ0 Handler						
.org 0x0004 jmp INTERRUPT_ERROR ; EXT_INT1 		; IRQ1 Handler						
.org 0x0006 jmp INTERRUPT_ERROR ; PCINT0 		; PCINT0 Handler					
.org 0x0008 jmp INTERRUPT_ERROR ; PCINT1 		; PCINT1 Handler					
.org 0x000A jmp INTERRUPT_ERROR ; PCINT2 		; PCINT2 Handler					
.org 0x000C jmp WDT 			; Watchdog Timer Handler							
.org 0x000E jmp INTERRUPT_ERROR ; TIM2_COMPA 	; Timer2 Compare A Handler			
.org 0x0010 jmp INTERRUPT_ERROR ; TIM2_COMPB 	; Timer2 Compare B Handler			
.org 0x0012 jmp TIM2_OVF 		; Timer2 Overflow Handler							
.org 0x0014 jmp INTERRUPT_ERROR ; TIM1_CAPT 	; Timer1 Capture Handler			
.org 0x0016 jmp INTERRUPT_ERROR ; TIM1_COMPA 	; Timer1 Compare A Handler			
.org 0x0018 jmp INTERRUPT_ERROR ; TIM1_COMPB 	; Timer1 Compare B Handler			
.org 0x001A jmp INTERRUPT_ERROR ; TIM1_OVF 		; Timer1 Overflow Handler			
.org 0x001C jmp INTERRUPT_ERROR ; TIM0_COMPA 	; Timer0 Compare A Handler			
.org 0x001E jmp INTERRUPT_ERROR ; TIM0_COMPB 	; Timer0 Compare B Handler			
.org 0x0020 jmp INTERRUPT_ERROR ; TIM0_OVF 		; Timer0 Overflow Handler			
.org 0x0022 jmp INTERRUPT_ERROR ; SPI_STC 		; SPI Transfer Complete Handler		
.org 0x0024 jmp INTERRUPT_ERROR ; USART_RXC 		; USART, RX Complete Handler	
.org 0x0026 jmp INTERRUPT_ERROR ; USART_UDRE 	; USART, UDR Empty Handler			
.org 0x0028 jmp INTERRUPT_ERROR ; USART_TXC 	; USART, TX Complete Handler		
.org 0x002A jmp INTERRUPT_ERROR ; ADC 			; ADC Conversion Complete Handler	
.org 0x002C jmp INTERRUPT_ERROR ; EE_RDY 		; EEPROM Ready Handler				
.org 0x002E jmp INTERRUPT_ERROR ; ANA_COMP 		; Analog Comparator Handler			
.org 0x0030 jmp INTERRUPT_ERROR ; TWI 			; 2-wire Serial Interface Handler	
.org 0x0032 jmp INTERRUPT_ERROR ; SPM_RDY 		; Store Program Memory Ready Handler


; Bei unkorrekten Interrupt aufruf				
; ROT+GELB+GRUEN blinkt							
INTERRUPT_ERROR:
	LED_ROT_ON
	LED_GELB_ON
	LED_GRUEN_ON
	call WAIT_100ms
	LED_ROT_OFF
	LED_GELB_OFF
	LED_GRUEN_OFF
	call WAIT_100ms
rjmp INTERRUPT_ERROR
; Bei WDT										
; ROT+GELB blinkt								
WDT:
	LED_ROT_ON
	LED_GELB_ON
	LED_GRUEN_OFF
	call WAIT_100ms
	LED_ROT_OFF
	LED_GELB_OFF
	LED_GRUEN_OFF
	call WAIT_100ms
rjmp WDT
; Reset												
RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
	call WDR_INITIALISIERUNG	; WDR_INITIALISIERUNG			
	call SRAM_CLEAR				; SRAM_CLEAR					
	call STARTWERTE				; STARTWERTE					
	call PORT_CLEAR				; PORT_CLEAR					
	call PORT_INITIALISIERUNG
	call ADC_INI_REF_5V
	call TWI_INITIALISIERUNG	; TWI_INITIALISIERUNG			
	call TWI_OLED_INITIALISIERUNG
	call TIMER2_1ms_INITIALISIERUNG
	call PROGRAMM_CHECK			; PROGRAMM_CHECK				
	SEI							; Interrupts freigeben			
;-------------------------------------------------------------------------------
	LED_GRUEN_ON
	call WAIT_500ms
	call OLED_VOLL
	call WAIT_100ms
	call OLED_CLEAR

	ldi temp1,0
	call OLED_HELL_TEMP1
	call ANZEIGE_BEGRUESSUNG
	call OLED_REFRESH
	call WAIT_1s

	ldi temp1,255
	call OLED_HELL_TEMP1
	call WAIT_500ms


	call OLED_CLEAR
	call OLED_TEST_CLEAR
	call WAIT_1s


	call WAIT_TEST
	call WAIT_1s

	call OLED_CLEAR	
	call LINIE_TEST

	call WAIT_1s
	call OLED_CLEAR
	call WAIT_1s
;-------------------------------------------------------------------------------
LOOP:
	WDR
	call ADC_MESSEN_5V_TEMP1
	call ANZEIGE_HAUPT
	call PROGRAMM_CHECK
	call OLED_REFRESH
	call WAIT_10ms
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "ADC.asm"
.include "ANZEIGE.asm"
.include "ASCII.asm"
.include "GRAFIC_PUNKT.asm"
.include "GRAFIC_LINIE.asm"
.include "ZAHLEN.asm"
.include "TEXTE.asm"
.include "TIMER.asm"
.include "TWI.asm"
.include "TWI_OLED.asm"
.include "SONSTIGES.asm"
.include "WAIT.asm"
.include "TABELLE_FONT_08x12.asm"
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEXT_BEGRUESSUNG_1:
.db "OLED ATmega328p",0
TEXT_BEGRUESSUNG_2:
.db "01/2021 V7 ",0
TEXT_BEGRUESSUNG_3:
.db "5x16=80 ASCII",0
TEXT_BEGRUESSUNG_4:
.db "128x64 Pixel ",0
TEXT_BEGRUESSUNG_5:
.db "8 MHz",0

TEXT_WAIT:
.db "wait 1s",0

TEXT_CLEAR:
.db "Clear",0
